/*
 *  ESFlashView.h
 *  Flash'In'App
 *
 *  Copyright 2009 Eltima Software. All rights reserved.
 *
 *	Flash'In'App is not free since version 2.0.
 *	If you plan to integrate it in your project, please, purchase a license at http://mac.eltima.com/flashinapp.html
 *	Questions? Please, contact Eltima at sales@eltima.com or visit http://mac.eltima.com for more info
 *  Full usage and integration guide can be found here: http://wiki.eltima.com/user-guides/flash-in-app-macos.html
 */

#import <Cocoa/Cocoa.h>
#import "ESScriptingPluginView.h"

/*
 * playback quality constants
 */
typedef enum _FlashMovieQuality
{
    FlashMovieQuality_Low
  , FlashMovieQuality_Medium
  , FlashMovieQuality_High
} FlashMovieQuality;

typedef struct _NPSignedRect
{
  int top;
  int left;
  int bottom;
  int right;
} NPSignedRect;

enum
{
    WMODE_WINDOW
  , WMODE_OPAQUE
  , WMODE_TRANSPARENT
};

/*
 * scale type constants
 */
typedef enum _FlashScaleType
{
    SCALE_SHOWALL
  , SCALE_NOBORDER
  , SCALE_NOSCALE
  , SCALE_EXACTFIT
} FlashScaleType;

/*
 * salign type constants
 */
typedef enum _FlashSAlignType
{
    SALIGN_DEFAULT
  , SALIGN_LEFT
  , SALIGN_TOP
  , SALIGN_RIGHT
  , SALIGN_BOTTOM
  , SALIGN_TOP_LEFT
  , SALIGN_TOP_RIGHT
  , SALIGN_BOTTOM_LEFT
  , SALIGN_BOTTOM_RIGHT
} FlashSAlignType;

@interface ESFlashView : ESScriptingPluginView
{
  NPSignedRect currentRect, originalRect, maxRect;
  BOOL canDrag;
  BOOL inDrag;
  NSPoint dragPoint;

	BOOL allowNativeDrag;
  BOOL enableSecurity;
	
  BOOL autoReadSWFInfo;
  int swfVersion;
  float frameRate;
  
  NSColor *flashBgColor;
  int wmode;
  NSString *flashVars;
  FlashScaleType scale;
  FlashSAlignType salign;
  NSArray *embedKeys;
  NSArray *embedValues;
}

/*
 * set/get Flash'In'App registration name and key
 * IMPORTANT! setName:andKey: method has to be called before loading any movie to register Flash'In'App, otherwise a demo reminder will be shown to user
 * NOTE: these methods are deprecated since Flash'In'App 2.5, please use [[ESFIAPreferences sharedPreferences] setName:andKey:]
 */
+ (void) setName:(NSString *)aName andKey:(NSString *)aKey;
+ (NSString *) registrationName;
+ (NSString *) registrationKey;

/*
 * set/get path to the plugin
 * by default plugin will be loaded from Contents/PlugIns/Flash Player.plugin
 
 * pluginPath - returns current path to the plugin
 * setPluginPath: - sets the new value for this setting (cannot be nil and should point to existing path)
 * NOTE: these methods are deprecated since Flash'In'App 2.5, please use [[ESFIAPreferences sharedPreferences] setFlashPluginPath:]
 */
+ (NSString *) pluginPath;
+ (void) setPluginPath:(NSString*)thePluginPath;

/*
 * turn transparency on/off
 * setTransparent: method has to be called before loading any movie
 * default value is "NO"
 * NOTE: should you change the transparency settings in runtime, you should reload the movie (loadMovie:atURL:withBaseURL:) 
 */
- (void)setTransparent:(BOOL)transparent;

/*
 * set/get Flash background color for playback
 * setFlashBackgroundColor: method has to be called before loading any movie
 * NOTE: should you change the background color settings in runtime, you should reload the movie (loadMovie:atURL:withBaseURL:) 
 */
- (void) setFlashBackgroundColor:(NSColor *)aColor;
- (NSColor *) flashBackgroundColor;

/*
 * This method lets you set root level variables to pass to Flash Player
 * setFlashVars: method has to be called before loading any movie
 * NOTE: should you set root level variables in runtime, you should reload the movie (loadMovie:atURL:withBaseURL:) 
 */
- (void) setFlashVars:(NSString *)aVars;
- (NSString *) flashVars;

/*
 * This method lets you specify scaling option used to display the Flash movie (see user guide for possible values)
 * setScaleType: method has to be called before loading any movie
 * NOTE: should you be changing scaling settings in runtime, you need reload the movie (loadMovie:atURL:withBaseURL:) 
 */
- (void) setScaleType:(FlashScaleType)aScale;
- (FlashScaleType) scaleType;

/*
 * set vertical and horizontal movie alignment within the player region (see user guide for possible values)
 * setSAlignType: method has to be called before loading any movie
 * NOTE: should you be changing movie alignment settings in runtime, you will need to reload the movie (loadMovie:atURL:withBaseURL:) 
 */
- (void) setSAlignType:(FlashSAlignType)anAlign;
- (FlashSAlignType) sAlignType;

/*
 * set all the rest of parameters provided by Adobe using this method (menu, quality, etc, see Adobe help pages)
 * setAttributeKeys:andValues: method has to be called before loading any movie
 * NOTE: should you be changing movie parameters in runtime, you will need to reload the movie (loadMovie:atURL:withBaseURL:) 
 */
- (void)setAttributeKeys:(NSArray *)keys andValues:(NSArray *)values;
- (NSArray *) attributeKeys;
- (NSArray *) attributeValues;

/*
 * turn FlashLocalSecurity on/off
 * must be called before loading movie
 * default and recommended value is "NO"

 * setEnabledSecurity: - sets the FlashLocalSecurity status to ON or OFF
 * isEnabledSecurity - returns current state of this setting
 */
- (void)setEnabledSecurity:(BOOL)value;
- (BOOL)isEnabledSecurity;

/*
 * allow/deny native dragging by plugin
 * zooming will be disabled, but flash components like sliders and scrolls will work

 * setAllowNativeDragg: - enables or disables handing dragging options by plugin
 * allowNativeDrag - returns current state of this setting
 */
- (void) setAllowNativeDrag:(BOOL)value;
- (BOOL) allowNativeDrag;

/*
 * globally allow/deny dragging by Flash'In'App, if it is enabled, allowNativeDrag setting will be ignored
 
 * canDrag - returns current state of this setting
 * setCanDrag: - enables or disables handling of dragging events by Flash'In'App

 *NOTE: this will work only if you correctly set current and original rects 
 */
- (BOOL) canDrag;
- (void) setCanDrag:(BOOL)value;

/*
 * functions to work with ZoomRect
 */
- (NPSignedRect) currentRect;
- (void) setCurrentRect:(NPSignedRect)rect;

- (NPSignedRect) originalRect;
- (void) setOriginalRect:(int32)left :(int32)top :(int32)right :(int32) bottom;

- (NPSignedRect) maxRect;

- (void) setOriginalSize:(BOOL)useViewSize;
- (void) setFitSize;

- (void) zoomIn;
- (void) zoomOut;

- (void) setZoomPercent:(int)value;
- (void) setZoomPercent:(int)value useViewSize:(BOOL)useViewSize;
- (int) zoomPercent;
- (int) zoomPercent:(BOOL)useViewSize;

- (void) displayRect:(NPSignedRect)rect;
- (void) displayZoomRect:(NSRect)rect;
- (void) updateCurrentRect;

/*
 * take movie snapshot in original size
 */
- (NSImage *) snapshotOriginalSize;

/*
 * scripting support
 
 * please, see the description of each method at Adobe's website: 
 * http://www.adobe.com/support/flash/publishexport/scriptingwithflash/scriptingwithflash_03.html
 */
- (BOOL) isPlaying;
- (void) play;
- (void) stopPlay;
- (int) totalFrames;
- (int) currentFrame;
- (void) gotoFrame:(int32)frame;
- (void) rewind;
- (void) back;
- (void) forward;
- (void) pan:(int32)x :(int32)y :(int32)panMode;
- (void) zoom:(int32)value;
- (void) setZoomRect:(int32)left :(int32)top :(int32)right :(int32) bottom;
- (FlashMovieQuality) flashQuality;
- (void) setFlashQuality:(FlashMovieQuality)value;

/*
 * Calls function with ExternalAPI help.
 * value – name and arguments of calling function, that has view <invoke name="asFunc"><arguments><string> Hello from Cocoa </string></arguments></invoke>
 */
- (id) callFunction:(NSString *)value;

- (void) setVariable:(NSString *)name toValue:(id)value;
- (id) getVariable:(NSString *)name;

- (int) swfVersion;

- (float)frameRate;

@end

@interface NSObject (ESFlashViewDelegate)
/*
 * this message will be sent to delegate when dragging is active

 * this is the method which you should use to realize dragging of the whole movie window when user drags it
 * (you can see the sample of how this works in our GUI sample)
 */
- (void) flashView:(ESFlashView *)flashView wasDraggedFrom:(NSPoint)fromPoint to:(NSPoint)toPoint;

/*
 * this message will be sent to delegate when movie calls an FSCommand
 * commandName - command name
 * arguments - arguments
 */
- (id) flashView:(ESFlashView *)flashView invokeFSCommand:(NSString *)commandName withArguments:(NSArray *)arguments;

/*
 * Informs the delegate that swf file performed ExternalInterface.call
 * xmlString - function name and arguments are represented as <invoke name="asFunc"><arguments><string>Hello from Cocoa</string></arguments></invoke>
 * Return value: Since Flash'In'App 2.5, you should return object (e.g. NSString), not xml string as it was before 
 * NOTE: if you implement flashView:callFunction: and flashView:callFunction:withArguments: methods, only second one will be called
 */
- (id) flashView:(ESFlashView *)flashView callFunction:(NSString *)xmlString;

/* 
 * Informs the delegate that swf file performed ExternalInterface.call
 * aName - function name that was called
 * anArguments - arguments of function call
 * Return value: Since Flash'In'App 2.5, you should return object (e.g. NSString), not xml string as it was before
 * NOTE: if you implement flashView:callFunction: and flashView:callFunction:withArguments: methods, only second one will be called
 */
- (id) flashView:(ESFlashView *)flashView callFunction:(NSString *)aName withArguments:(NSArray *)anArguments;

/*
 * This message will be sent to delegate when ESFlashView change visible rect of movie.
 */
- (void) flashView:(ESFlashView *)flashView didDisplayRect:(NPSignedRect)aDisplayedRect;
 
@end
